#!/bin/sh
#
# This file was produced by running the Configure script. It holds all the
# definitions figured out by Configure. Should you modify one of these values,
# do not forget to propagate your changes by running "Configure -der". You may
# instead choose to run each of the .SH files by yourself, or "Configure -S".
#

# Configuration time: Fri Nov 17 19:50:28 GMT 2000
# Configured by: mcv21
# Target system: linux pick 2.2.14 #1 wed jan 12 13:49:58 gmt 2000 i686 unknown 

d_bsd='undef'
d_eunice='undef'
d_xenix='undef'
eunicefix=':'
Mcc='Mcc'
awk='awk'
basename='basename'
bash='bash'
cat='cat'
cp='cp'
cpp='cpp'
csh='csh'
date='date'
diff='diff'
echo='echo'
ed='ed'
egrep='egrep'
expr='expr'
grep='grep'
inews='inews'
ispell='ispell'
ksh='ksh'
less='less'
lint=''
mail='mail'
metamail='metamail'
mhn='mhn'
mkdir='mkdir'
more='more'
munpack='munpack'
mv='mv'
nroff='nroff'
pg='pg'
rm='rm'
rmail='rmail'
sed='sed'
sendmail='sendmail'
smail='smail'
sort='sort'
spell='spell'
tail='tail'
test='test'
tr='tr'
uname='uname'
uniq='uniq'
uuname='uuname'
vi='vi'
vspell=''
who=''
hint='previous'
myuname='linux pick 2.2.14 #1 wed jan 12 13:49:58 gmt 2000 i686 unknown '
srcdir='.'
vincludes=''
vpath='.'
Id='$Id'
Log='$Log'
active='none'
acttimes='nntp'
d_acttimes='define'
myactive='.falseactive'
bin='/usr/lib/trn'
binexp='/usr/lib/trn'
installbin='/usr/lib/trn'
cc='gcc'
ccflags=''
cppflags=''
ldflags='-s'
lkflags=''
optimize='-O3'
cf_by='mcv21'
cf_time='Fri Nov 17 19:50:28 GMT 2000'
contains='grep'
cpplast='-'
cppminus='-'
cpprun='gcc -E'
cppstdin='gcc -E'
d_ftime='define'
aphostcmd=''
d_gethname='undef'
d_phostcmd='undef'
d_uname='undef'
d_getpwent='define'
d_getcwd='undef'
d_getwd='define'
d_havetlib='define'
termlib='-lcurses'
d_ignoreorg='undef'
d_internet='define'
d_memcmp='define'
d_memcpy='define'
d_memset='define'
d_mimeshow='undef'
d_mimestore='undef'
mimeshow="none"
mimestore="none"
d_newsadm='define'
newsadmin='news'
d_nntp='define'
d_xdata='define'
d_genauth='define'
servername='/etc/news/server'
d_nolnbuf='undef'
d_normsig='define'
jobslib=''
d_novoid='undef'
void=''
d_portable='define'
d_rdchk='undef'
d_rename='define'
d_sigblock='define'
d_sighold='define'
d_sizet='undef'
d_strccmp='define'
d_strchr='define'
d_strftime='define'
strftimec=''
strftimeo=''
d_libndir='undef'
d_usendir='undef'
libndir=''
ndirc=''
ndiro=''
d_vfork='define'
d_voidsig='define'
signal_t='void'
defeditor='/usr/bin/editor'
filexp='/usr/share/trn/filexp'
d_dirnamlen=''
i_dirent='define'
i_ptem='undef'
i_stdlib='define'
i_string='define'
strings='/usr/include/string.h'
i_sysdir='define'
d_voidtty=''
i_bsdioctl=''
i_sysfilio='undef'
i_sysioctl='define'
i_syssockio=''
i_sysndir='undef'
i_sgtty='undef'
i_termio='undef'
i_termios='define'
i_systime='undef'
i_time='define'
i_unistd='define'
i_vfork='undef'
ispell_options=''
ispell_prg='/usr/bin/ispell'
libc='/usr/lib/libc.a'
plibpth='/usr/ccs/lib /usr/lib /usr/ucblib /usr/local/lib'
xlibpth='/usr/lib/386 /lib/386'
libs=' '
citydist='none'
cntrydist='none'
contdist='none'
locdist='none'
multistatedist='none'
orgdist='none'
statedist='none'
mailer='/usr/sbin/sendmail'
mailfile='/var/spool/mail/%L'
installmansrc='/usr/share/man/man1'
manext='1'
mansrc='/usr/share/man/man1'
mansrcexp='/usr/share/man/man1'
mboxchar='F'
c=''
n='-n'
d_berknames='define'
d_passnames='define'
d_usgnames='undef'
nametype='bsd'
inewsloc='inews'
newslib='/tmp'
newslibexp='/tmp'
newsspool='/tmp'
orgname='/etc/news/organization'
package='trn'
spackage='Trn'
pager='/usr/bin/pager'
phost='/etc/news/whoami'
phostcmd=''
prefshell='/bin/sh'
installprivlib='/usr/share/trn'
privlib='/usr/share/trn'
privlibexp='/usr/share/trn'
rootid='0'
sharpbang='#!'
shsharp='true'
spitshell='cat'
startsh='#!/bin/sh'
sysman='/usr/man/man1'
d_usemt='define'
d_useov='define'
overviewdir='/tmp'
threaddir='/tmp'
trn_init='FALSE'
trn_select='TRUE'
nm_opt=''
runnm='true'
usenm='true'
incpath=''
mips=''
mips_type=''
usrinc='/usr/include'
PATCHLEVEL="3.6 (20 Nov 1994)"
CONFIG=true
